<?php defined('BASEPATH') OR exit('No direct script access allowed');
require_once(APPPATH . 'libraries/Stripe/lib/Stripe.php');

class Invoices extends Application_controller {
	
	function __construct() {
		
		parent::__construct();
		initial_data_contoller($this);

		if(! (logged_user()->isOwner() || !logged_user()->isMember()) ) redirect('dashboard');
				
	}
	
	public function index() {
		
		if(logged_user()->isOwner()) {
			$invoices = $this->Invoices->getAll();
		} else {
			$invoices = $this->Invoices->getByClientOnly(logged_user());
		}
		
		tpl_assign('invoices', $invoices);
	
	}

	private function validate($access_key) {

		$invoice = $this->Invoices->getByAccessKey($access_key);
		if(is_null($invoice) || (!logged_user()->isMember() 
		&& logged_user()->getId() != $invoice->getClientId()) ) {
		
			set_flash_error(lang('e_3'));
			redirect('dashboard');
		
		}
		
		return $invoice;
	
	}
	
	public function view($access_key) {

		$this->setLayout("page_dialog");
		
		$invoice = $this->validate($access_key);		
		tpl_assign('invoice', $invoice);

		$paymentmode = input_post_request('paymentmode');
		tpl_assign("paymentmode", $paymentmode);

		$amount = (float) input_post_request('amount');
		tpl_assign("amount", $amount);

		$is_submited = input_post_request('submited') ==  'submited';
		tpl_assign("is_submited", $is_submited);

		if($is_submited && $amount <= 0) {

			set_flash_error(lang('c_370'));
			redirect($invoice->getObjectURL());

		}

	}

	public function payment($gateway, $access_key) {

		$this->setLayout("page_dialog");
		$invoice = $this->validate($access_key);
				
		switch($gateway) {
		
			case 'stripe' :

				$amount = (float) input_post_request('amount');
				$access_token = input_post_request('stripeToken');

				$stripe_secret_key = config_option('stripe_secret_key'); 
				$stripe_currency_code = config_option('stripe_currency_code'); 

				try {
				
					Stripe::setApiKey($stripe_secret_key);
					$charge = Stripe_Charge::create(array(
								"amount" => round(($amount)*100),
								"currency" => $stripe_currency_code,
								"card" => $access_token,
								"description" => "Payment for Invoice ".$invoice->getInvoiceNo()
					));

					try{
						
						$this->db->trans_begin();
	
						$transaction_log = new TransactionLog();
	
						$transaction_log->setAmount($amount);
						$transaction_log->setTransactionType("payment");
						$transaction_log->setReferenceId($invoice->getId());
						$transaction_log->setDescription("Stripe payment (".$charge->id.")");
						
						$transaction_log->setCreditAccountId($invoice->getClientId());
						$transaction_log->setDebitAccountId(owner_company()->getCreatedById());
						
						$transaction_log->save();
						
						if(!$invoice->getStatus()) $invoice->setStatus(true); // Reopen
	
						$total_paid_amount = $invoice->getPaidAmount() + $amount;
						$invoice->setPaidAmount($total_paid_amount);
							
						$invoice->save();
																	
						$this->db->trans_commit();

						set_flash_success(sprintf(lang('c_79'), lang('c_449')));
														
					} catch (Exception $e) {
					
						set_flash_error(lang('e_3'));
						$this->db->trans_rollback();
					
					}

				} catch (Stripe_CardError $e) {
					set_flash_error(STRIPE_FAILED);
				} catch (Stripe_InvalidRequestError $e) {
					set_flash_error($e->getMessage());
				} catch (Stripe_AuthenticationError $e) {
					set_flash_error(AUTHENTICATION_STRIPE_FAILED);
				} catch (Stripe_ApiConnectionError $e) {
					set_flash_error(NETWORK_STRIPE_FAILED);
				} catch (Stripe_Error $e) {
					set_flash_error(STRIPE_FAILED);
				} catch (Exception $e) {
					set_flash_error(STRIPE_FAILED);
				}
			
			break;
			
			default:
				
				set_flash_error(lang('e_3'));
				
			break;		

		}		

		redirect($invoice->getObjectURL());
		
	}
		
	public function create($request_project_id = null) {

		if(!logged_user()->isOwner()) die();
		only_ajax_request_allowed();

		$this->setLayout('modal');
		$this->setTemplate('invoices/_invoice_form');

		$invoice = new Invoice();
		tpl_assign('invoice', $invoice);

		$client_users = $this->Users->getByClients();
		tpl_assign('client_users', $client_users);

		$projects = $this->Projects->getAll();
		tpl_assign('projects', $projects);

		$subject = input_post_request('subject');
		tpl_assign("subject", $subject);

		$client_id = input_post_request('client_id');
		tpl_assign("client_id", $client_id);

		$project_id = input_post_request('project_id', $request_project_id);
		tpl_assign("project_id", $project_id);
	
		$status = input_post_request('status', 1);
		tpl_assign("status", $status);

		$currency = input_post_request('currency');
		tpl_assign("currency", $currency);

		$due_date = input_post_request('due_date');
		tpl_assign("due_date", $due_date);

		$tax = input_post_request('tax');
		tpl_assign("tax", $tax);

		$tax_rate = input_post_request('tax_rate');
		tpl_assign("tax_rate", $tax_rate);

		$items = input_post_request('items', 0);
		$is_submited = input_post_request('submited') ==  'submited';
		
		if ($is_submited) {

			$this->form_validation->set_rules('subject', lang('c_126'), 'trim|required|max_length[100]');
			$this->form_validation->set_rules('client_id', lang('c_29'), 'required|greater_than[0]',  array('greater_than' => lang('c_129')));
					
			if ($this->form_validation->run() == FALSE) {
				$this->renderText(output_ajax_request(false, validation_errors()));
			} else {
		
				try{
					
					$this->db->trans_begin();

					$invoice->setClientId($client_id);
					$invoice->setProjectId($project_id);

					$invoice->setStatus($status == 1);

					$invoice->setSubject($subject);
					$invoice->setDueDate($due_date);

					$invoice->setTax($tax);
					$invoice->setTaxRate($tax_rate);

					$invoice->setCreatedById(logged_user()->getId());

					$invoice->save();
					
					$sub_total=0;
					$o_key = 0;
					
					for ($i=1;$i<=$items;$i++) {
					
						$delete = intval(input_post_request("delete_" . $i));
						if (!$delete) {

							$name = input_post_request("name_" . $i);
							$quantity = abs(input_post_request("quantity_" . $i));
							$amount = abs(input_post_request("amount_" . $i));

							if ($quantity > 0 && $amount > 0 && $name != "") {
								
								$invoice_item = new InvoiceItem();

								$o_key++;
								$invoice_item->setOKey($o_key);
								
								$item_hash_id = sha1(uniqid(rand(), true).$invoice->getId().
								$quantity.$amount.$name.$o_key.time());
								
								$invoice_item->setUKey($item_hash_id);

								$invoice_item->setInvoiceId($invoice->getId());
								$invoice_item->setQuantity($quantity);
								$invoice_item->setAmount($amount);
								$invoice_item->setDescription($name);
												
								$invoice_item->save();
												
								$sub_total += $amount*$quantity;
							
							}
							
						}
					}

					if ($tax_rate > 0) {
					
						$tax_amount = abs($sub_total/100*$tax_rate);
						$sub_total += $tax_amount;
					
					}					

					$invoice->setTotalAmount($sub_total);
					
					$invoice->setInvoiceNo("INV-".str_pad($invoice->getId(), 6, '0', STR_PAD_LEFT));
					$invoice->setAccessKey(sha1(uniqid(rand(), true).$invoice->getId().time()));
					
					$invoice->save();

					$this->db->trans_commit();
					
					// Send notification ..
					$email_to_client = intval(input_post_request("email_to_client"));
					if($email_to_client) {
 
						$notify_subject = lang('c_171');
						$notify_message = $this->load->view("emails/invoice", 
						array("invoice_subject" => $invoice->getSubject(), "invoice_no" => $invoice->getInvoiceNo(), 
						"invoice_link" => get_page_base_url($invoice->getObjectURL())), true);

						$notify_user_object = $this->Users->findById($client_id);
						if(isset($notify_user_object)) {
							try {
								$this->UserNotifications->create($notify_user_object, $notify_subject, $notify_message);
								send_mail_SMTP($notify_user_object->getEmail(), $notify_user_object->getName(), $notify_subject, $notify_message);
							} catch (Exception $e){}
						}
							
					}
					
					set_flash_success(sprintf(lang('c_128'), lang('c_173')));
		
				} catch (Exception $e) {

					$this->db->trans_rollback();
					set_flash_error(lang('e_1'));

				}

				$this->renderText(output_ajax_request(true));

			}

		}
			
	}
	
	public function edit($id) {

		if(!logged_user()->isOwner()) die();
		only_ajax_request_allowed();

		$this->setLayout('modal');
		$this->setTemplate('invoices/_invoice_form');

		$invoice = $this->Invoices->findById($id);
		if(is_null($invoice)) set_flash_error(lang('e_3'), true);
		
		tpl_assign('invoice', $invoice);

		$client_users = $this->Users->getByClients();
		tpl_assign('client_users', $client_users);

		$projects = $this->Projects->getAll();
		tpl_assign('projects', $projects);

		$subject = input_post_request('subject', $invoice->getSubject());
		tpl_assign("subject", $subject);

		$client_id = input_post_request('client_id', $invoice->getClientId());
		tpl_assign("client_id", $client_id);

		$project_id = input_post_request('project_id', $invoice->getProjectId());
		tpl_assign("project_id", $project_id);
	
		$status = input_post_request('status', (int) $invoice->getStatus());
		tpl_assign("status", $status);

		$due_date_timestamp = $invoice->getDueDate();
		$formatted_due_date = $due_date_timestamp ? format_date($due_date_timestamp, 'm/d/Y') : null;
		
		$due_date = input_post_request('due_date', $formatted_due_date);
		tpl_assign("due_date", $due_date);

		$tax = input_post_request('tax', $invoice->getTax());
		tpl_assign("tax", $tax);

		$tax_rate = input_post_request('tax_rate', $invoice->getTaxRate());
		tpl_assign("tax_rate", $tax_rate);

		$items = input_post_request('items', 0);
		$is_submited = input_post_request('submited') ==  'submited';

		if ($is_submited) {

			$this->form_validation->set_rules('subject', lang('c_126'), 'trim|required|max_length[100]');
			$this->form_validation->set_rules('client_id', lang('c_29'), 'required|greater_than[0]',  array('greater_than' => lang('c_129')));
					
			if ($this->form_validation->run() == FALSE) {
				$this->renderText(output_ajax_request(false, validation_errors()));
			} else {
		
				try{
					
					$this->db->trans_begin();

					$invoice->setClientId($client_id);
					$invoice->setProjectId($project_id);

					$invoice->setStatus($status == 1);

					$invoice->setSubject($subject);
					$invoice->setDueDate($due_date);

					$invoice->setTax($tax);
					$invoice->setTaxRate($tax_rate);
					
					$invoice->clearInvoiceItems();

					$sub_total=0;
					$o_key = 0;
					
					for ($i=1;$i<=$items;$i++) {
					
						$delete = intval(input_post_request("delete_" . $i));
						if (!$delete) {

							$name = input_post_request("name_" . $i);
							$quantity = abs(input_post_request("quantity_" . $i));
							$amount = abs(input_post_request("amount_" . $i));

							if ($quantity > 0 && $amount > 0 && $name != "") {
								
								$invoice_item = new InvoiceItem();

								$o_key++;
								$invoice_item->setOKey($o_key);
								
								$item_hash_id = sha1(uniqid(rand(), true).$invoice->getId().
								$quantity.$amount.$name.$o_key.time());
								
								$invoice_item->setUKey($item_hash_id);

								$invoice_item->setInvoiceId($invoice->getId());
								$invoice_item->setQuantity($quantity);
								$invoice_item->setAmount($amount);
								$invoice_item->setDescription($name);
												
								$invoice_item->save();
												
								$sub_total += $amount*$quantity;
							
							}
							
						}
					}

					if ($tax_rate > 0) {
					
						$tax_amount = abs($sub_total/100*$tax_rate);
						$sub_total += $tax_amount;
					
					}					

					$invoice->setTotalAmount($sub_total);
					$invoice->save();

					$this->db->trans_commit();
					
					// Send notification ..
					$email_to_client = intval(input_post_request("email_to_client"));
					if($email_to_client) {
 
						$notify_subject = lang('c_171');
						$notify_message = $this->load->view("emails/invoice", 
						array("invoice_subject" => $invoice->getSubject(), "invoice_no" => $invoice->getInvoiceNo(), 
						"invoice_link" => get_page_base_url($invoice->getObjectURL())), true);

						$notify_user_object = $this->Users->findById($client_id);
						if(isset($notify_user_object)) {
							try {
								$this->UserNotifications->create($notify_user_object, $notify_subject, $notify_message);
								send_mail_SMTP($notify_user_object->getEmail(), $notify_user_object->getName(), $notify_subject, $notify_message);
							} catch (Exception $e){}
						}
							
					}
					
					set_flash_success(sprintf(lang('c_57'), lang('c_173')));
		
				} catch (Exception $e) {

					$this->db->trans_rollback();
					set_flash_error(lang('e_1'));

				}

				$this->renderText(output_ajax_request(true));

			}
			
		} 
			
	}
	
}
