<?php defined('BASEPATH') OR exit('No direct script access allowed');

/* Invoices Model */

class Invoices_model extends Application_model {

    function __construct() {
        parent::__construct('invoices', 'Invoice');
    }

	public function getAll($include_trashed = false) {

		return $this->find(array('conditions' => array('`is_trashed` <= ?', $include_trashed), 
		'order' => '`id` DESC'));
						
	}

	public function getByClientOnly(User $client, $include_trashed = false) {
		
		if(!$client->isMember()) {

			return $this->find(array('conditions' => array('`is_trashed` <= ? AND `client_id` = ?', $include_trashed, $client->getId()),
			'order' => '`id` DESC'));
		}
		
		return null;		
	}

	public function getAllOpen() {

		return $this->find(array('conditions' => array('`status` >= ? AND `is_trashed` <= ?', 1, false), 
		'order' => '`id` DESC'));
						
	}

	public function getByProject(Project $project, $include_trashed = false) {

		return $this->find(array('conditions' => array('`is_trashed` <= ? AND `project_id` = ?', $include_trashed, $project->getId()), 
		'order' => '`created_at` DESC'));
						
	}

	public function getByAccessKey($access_key) {
		return  $this->find(array('conditions' => array('`access_key` = ?', $access_key), 'one' => true));
	}

	public function getByTrashed() {
		return $this->find(array('conditions' => array('`is_trashed` = ?', true), 'order' => '`id`'));
	}

	public function getOverdues($client_ids) {
		return $this->find(array('conditions' => array('`is_trashed` = ? AND `status` = ? AND `client_id` IN (?) AND `due_date` < NOW() AND `total_amount` >= `paid_amount`', false, true, $client_ids), 'order' => '`id`'));
	}

}

/* Invoice Object */

class Invoice extends Application_object {

	private $client;
	private $project;

	function __construct() {
		parent::__construct('invoices');
	}

	public function getName() {
		return $this->getInvoiceNo();
	}

	public function getClient() {
		
		if(is_null($this->client)) {
			$this->client = $this->CI_instance()->Users->findById($this->getClientId());	
		}
		
		return $this->client;
		
	}

	public function getProject() {
		
		if(is_null($this->project)) {
			$this->project = $this->CI_instance()->Projects->findById($this->getProjectId());	
		}
		
		return $this->project;
		
	}

	public function moveToTrash() {
		
		$this->setIsTrashed(true);
		$this->save();
			
	}

	public function restoreFromTrash() {
		
		$this->setIsTrashed(false);
		$this->save();

	}

	public function getItems() {
		return $this->CI_instance()->InvoiceItems->getItems($this);
	}

	public function getItemsCount() {
		return $this->CI_instance()->InvoiceItems->countItems($this);
	}

	public function clearInvoiceItems() {
		
		$items = $this->getItems();
		if(isset($items) && is_array($items) && count($items)) {
			foreach($items as $item) $item->delete();	
		}
			
	}

	public function getPayments() {
		return $this->CI_instance()->TransactionLogs->getReferenceLogs("payment", $this->getId());
	}

	private function clearPayments() {
		
		$payments = $this->getPayments();
		if(isset($payments) && is_array($payments) && count($payments)) {
			foreach($payments as $payment) $payment->delete();	
		}
			
	}

	public function delete() {
		
		$this->clearInvoiceItems();
		$this->clearPayments();
		
		parent::delete();
		
	}
	

	/* Static URLs */

    function getCreatePaymentURL() {
		return 'transactions/create/payment/'.$this->getId();
	}

    function getEditURL() {
		return 'invoices/edit/'.$this->getId();
	}

    function getPaymentURL($gateway) {
		return 'invoices/payment/'.$gateway.'/'.$this->getAccessKey();
	}

    function getObjectURL() {
		return 'invoices/view/'.$this->getAccessKey();
	}

}