<?php defined('BASEPATH') OR exit('No direct script access allowed');

/* ProjectTimelogs Model */

class ProjectTimelogs_model extends Application_model {

    function __construct() {
        parent::__construct('project_timelogs', 'ProjectTimelog');
    }

	public function getAll($include_trashed = false, $status = true) {

		return $this->find(array('conditions' => array('`is_trashed` <= ? AND `is_approved` = ?', $include_trashed, $status), 
		'order' => '`id` DESC'));

	}

	public function getByProjects($project_ids, $include_trashed = false) {

		if(logged_user()->isOwner() || logged_user()->isAdmin() || !logged_user()->isMember()) {
		
			return $this->find(array('conditions' => array('`is_trashed` <= ? AND `project_id` IN (?)', $include_trashed, $project_ids), 
			'order' => '`id` DESC'));
		
		} else {

			return $this->find(array('conditions' => array('`is_trashed` <= ? AND `project_id` IN (?) AND `member_id` = ? ', $include_trashed, $project_ids, logged_user()->getId()), 
			'order' => '`id` DESC'));
		
		}
								
	}

	public function getByUser(User $member, $include_trashed = false, $status = true) {

		return $this->find(array('conditions' => array('`is_trashed` <= ? AND `member_id` = ? AND `is_approved` = ?', $include_trashed, $member->getId(), $status), 
		'order' => '`id` DESC'));
						
	}

	public function getTimelogStats($user_ids, $start = null, $end = null) {
	
		$project_timelogs_table = $this->getTableName(true);
		
		$query = $this->db->query("SELECT SUM(`total_hours`) AS `summary`, `member_id`, `project_id` FROM ".$project_timelogs_table."
		WHERE `is_approved` = 1".(isset($start) && isset($end) ? " AND DATE(`created_at`) BETWEEN '$start' AND '$end'" : "")." 
		AND `member_id` IN (".implode(",", $user_ids).") GROUP BY `member_id`, `project_id` HAVING `summary` > 0");
		
		return $query->result();

	}

	public function getTimelogAllStats() {
	
		$project_timelogs_table = $this->getTableName(true);
		
		$query = $this->db->query("SELECT SUM(`total_hours`) AS `summary` FROM ".$project_timelogs_table." 
		WHERE `is_approved` = 1");
		
		return $query->result();

	}

	public function getByTrashed() {
		return $this->find(array('conditions' => array('`is_trashed` = ?', true), 'order' => '`id`'));
	}

}

/* ProjectTimelog Object */

class ProjectTimelog extends Application_object {

	private $member;

	function __construct() {
		parent::__construct('project_timelogs');
	}

	public function getName() {
		return shorter($this->getMemo(), 65);
	}

	public function getMember() {

		if(is_null($this->member)) {
			$this->member = $this->CI_instance()->Users->findById($this->getMemberId());	
		}
		
		return $this->member;
		
	}

	public function moveToTrash() {
		
		$this->setIsTrashed(true);
		$this->save();
			
	}

	public function restoreFromTrash() {
		
		$this->setIsTrashed(false);
		$this->save();

	}

	/* Static URLs */

    function getObjectURL() {
		return false;
	}

    function getEditURL() {
		return 'timesheet/edit/'.$this->getId();
	}

}